from __future__ import annotations

from dataclasses import dataclass
from typing import Callable

import gradio as gr

from grote.collections.base import COMPONENT_CONFIGS, ComponentCollection, buildmethod
from grote.collections.translate import TranslateComponents
from grote.functions import check_and_parse_inputs_fn, initialize_translate_interface_fn, record_trial_start_fn

LOAD_CFG = COMPONENT_CONFIGS["load"]


@dataclass
class LoadComponents(ComponentCollection):
    _id: str = "load"

    login_code_description_cap: gr.Markdown = None
    login_code_txt: gr.Textbox = None
    input_description_cap: gr.Markdown = None
    file_in: gr.File = None
    sentences_txt: gr.Textbox = None
    start_btn: gr.Button = None

    @classmethod
    def get_login_code_description_cap(cls, value: str | None = None, visible: bool = True) -> gr.Markdown:
        if not value:
            value = LOAD_CFG["login_code_description"]
        return gr.Markdown(value, visible=visible, elem_id="login_code_description_cap")

    @classmethod
    def get_login_code_txt(cls, value: str | Callable = "", visible: bool = True) -> gr.components.Textbox:
        return gr.Textbox(
            label=LOAD_CFG["login_code_label"],
            lines=1,
            elem_id="login_code_txt",
            placeholder=LOAD_CFG["login_code_placeholder"],
            value=value,
            visible=visible,
            info=LOAD_CFG["login_code_info"],
        )

    @classmethod
    def get_input_description_cap(cls, value: str | None = None, visible: bool = True) -> gr.Markdown:
        if not value:
            value = LOAD_CFG["input_description"]
        return gr.Markdown(value, visible=visible, elem_id="input_description_cap")

    @classmethod
    def get_file_in(cls, value: str | list[str] | Callable | None = None, visible: bool = True) -> gr.File:
        return gr.File(
            label=LOAD_CFG["file_label"],
            interactive=True,
            elem_id="file_in",
            file_count="single",
            file_types=["txt"],
            value=value,
            visible=visible,
        )

    @classmethod
    def get_sentences_txt(cls, value: str | Callable = "", visible: bool = False) -> gr.components.Textbox:
        return gr.Textbox(
            label=LOAD_CFG["sentences_label"],
            lines=6,
            elem_id="sentences_txt",
            placeholder=LOAD_CFG["sentences_placeholder"],
            value=value,
            visible=visible,
        )

    @classmethod
    def get_start_btn(cls, visible: bool = True) -> gr.Button:
        return gr.Button(LOAD_CFG["start_button_label"], variant="primary", elem_id="start_btn", visible=visible)

    @classmethod
    @buildmethod
    def build(cls: LoadComponents) -> LoadComponents:
        lc: LoadComponents = cls()
        lc.login_code_description_cap = lc.get_login_code_description_cap()
        lc.login_code_txt = lc.get_login_code_txt()
        lc.input_description_cap = lc.get_input_description_cap()
        lc.file_in = lc.get_file_in()
        lc.sentences_txt = lc.get_sentences_txt()
        lc.start_btn = lc.get_start_btn()
        return lc

    def set_listeners(self, tc: TranslateComponents, out_state: gr.State) -> None:
        self.start_btn.click(
            check_and_parse_inputs_fn,
            inputs=[
                self.login_code_txt,
                self.file_in,
                self.sentences_txt,
                self.state,
            ],
            outputs=[
                self.state,
            ],
        ).success(
            initialize_translate_interface_fn,
            inputs=[self.state, tc.state],
            outputs=[tc.textboxes_col]
            + self.components
            + tc.components
            + [c for c in tc.textboxes_col.children if isinstance(c, gr.Row)],
        ).success(
            record_trial_start_fn,
            inputs=[out_state, self.state],
            outputs=[out_state],
        )



from .load import check_and_parse_inputs_fn, initialize_translate_interface_fn
from .translate import (
    record_textbox_blur_fn,
    record_textbox_focus_fn,
    record_textbox_input_fn,
    record_textbox_remove_highlights_fn,
    record_trial_end_fn,
    record_trial_start_fn,
    save_outputs_to_file,
)

__all__ = [
    "check_and_parse_inputs_fn",
    "initialize_translate_interface_fn",
    "record_textbox_blur_fn",
    "record_textbox_focus_fn",
    "record_textbox_input_fn",
    "record_textbox_remove_highlights_fn",
    "record_trial_start_fn",
    "record_trial_end_fn",
    "save_outputs_to_file",
]



from datetime import datetime
from typing import Any

import gradio as gr

from grote.collections.base import COMPONENT_CONFIGS

TRANS_CFG = COMPONENT_CONFIGS["translate"]


def get_current_time() -> str:
    return datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]


def record_trial_start_fn(state: dict[str, Any], lc_state: dict[str, Any]) -> dict[str, Any]:
    out = {
        "time": get_current_time(),
        "login_code": lc_state["login_code_txt"],
        "event_type": "start",
    }
    state["events"].append(out)
    return state


def record_textbox_focus_fn(state: dict[str, Any], textbox_content: dict, lc_state: dict[str, Any]) -> dict[str, Any]:
    out = {
        "time": get_current_time(),
        "login_code": lc_state["login_code_txt"],
        "text_id": textbox_content["id"],
        "event_type": "enter",
    }
    state["events"].append(out)
    return state


def record_textbox_input_fn(state: dict[str, Any], textbox_content: dict, lc_state: dict[str, Any]) -> dict[str, Any]:
    current_text = "".join(
        f"<{tag_id.lower()}>{text}</{tag_id.lower()}>" if tag_id is not None else text
        for text, tag_id in textbox_content["data"]
    )
    if textbox_content["id"] not in state or current_text != state[textbox_content["id"]]:
        out = {
            "time": get_current_time(),
            "login_code": lc_state["login_code_txt"],
            "text_id": textbox_content["id"],
            "event_type": "change",
            "text": current_text,
        }
        state[textbox_content["id"]] = current_text
        state["events"].append(out)
    return state


def record_textbox_blur_fn(state: dict[str, Any], textbox_content: dict, lc_state: dict[str, Any]) -> dict[str, Any]:
    out = {
        "time": get_current_time(),
        "login_code": lc_state["login_code_txt"],
        "text_id": textbox_content["id"],
        "event_type": "exit",
    }
    state["events"].append(out)
    return state


def record_textbox_remove_highlights_fn(
    state: dict[str, Any], textbox_content: dict, lc_state: dict[str, Any]
) -> dict[str, Any]:
    current_text = "".join(
        f"<{tag_id.lower()}>{text}</{tag_id.lower()}>" if tag_id is not None else text
        for text, tag_id in textbox_content["data"]
    )
    out = {
        "time": get_current_time(),
        "login_code": lc_state["login_code_txt"],
        "text_id": textbox_content["id"],
        "event_type": "remove_highlights",
        "text": current_text,
    }
    state[textbox_content["id"]] = current_text
    state["events"].append(out)
    return state


def record_trial_end_fn(state: dict[str, Any], lc_state: dict[str, Any]) -> dict[str, Any]:
    gr.Info("Saving trial information. Don't close the tab until the download button is available!")
    out = {
        "time": get_current_time(),
        "login_code": lc_state["login_code_txt"],
        "event_type": "end",
    }
    state["events"].append(out)
    return state


def save_outputs_to_file(lc_state, *txts) -> None:
    fname = f"{lc_state['login_code_txt']}_{lc_state['_filename']}_output.txt"
    with open(fname, "w") as f:
        f.write("\n".join("".join(x[0] for x in txt["data"]) for txt in txts if txt["data"]))
    gr.Info("Saving complete! Download the output file by clicking the 'Download translations' button below.")
    return gr.DownloadButton(
        label=TRANS_CFG["download_button_label"],
        value=fname,
        visible=True,
        interactive=True,
    ), gr.Button(interactive=False)



from __future__ import annotations

import re
from dataclasses import dataclass
from typing import Any, Callable, Literal

import gradio as gr
from gradio_highlightedtextbox import HighlightedTextbox

from grote.collections.base import COMPONENT_CONFIGS, ComponentCollection, buildmethod
from grote.config import CONFIG as cfg
from grote.event_logging import EventLogger
from grote.functions import (
    record_textbox_blur_fn,
    record_textbox_focus_fn,
    record_textbox_input_fn,
    record_textbox_remove_highlights_fn,
    record_trial_end_fn,
    save_outputs_to_file,
)

TRANS_CFG = COMPONENT_CONFIGS["translate"]


@dataclass
class TranslateComponents(ComponentCollection):
    _id: str = "translate"

    source_side_label: gr.Markdown = None
    target_side_label: gr.Markdown = None
    target_side_legend: gr.Markdown = None
    reload_btn: gr.Button = None
    done_btn: gr.Button = None
    download_btn: gr.DownloadButton = None
    textboxes_col: gr.Column = None

    @property
    def textboxes(self) -> list[gr.Textbox | HighlightedTextbox]:
        return [c for c in self.components if isinstance(c, (gr.Textbox, HighlightedTextbox))]

    @property
    def target_textboxes(self) -> list[HighlightedTextbox]:
        return [
            c for c in self.components if isinstance(c, HighlightedTextbox) and re.match(r"target_\d+_txt", c.elem_id)
        ]

    @classmethod
    def get_source_side_label_cap(cls, value: str | None = None, visible: bool = False) -> gr.Markdown:
        if not value:
            value = TRANS_CFG["source_side_label"]
        return gr.Markdown(value, visible=visible, elem_id="source_side_label_cap")

    @classmethod
    def get_target_side_label_cap(cls, value: str | None = None, visible: bool = False, has_highlights: bool = True) -> gr.Markdown:
        if not value:
            value = TRANS_CFG["target_side_label"] if has_highlights else TRANS_CFG["target_side_label_no_highlights"]
        return gr.Markdown(value, visible=visible, elem_id="target_side_label_cap")

    @classmethod
    def get_target_side_legend_cap(cls, value: str | None = None, visible: bool = False, has_highlights: bool = True) -> gr.Markdown:
        if not value and cfg.tag_labels and cfg.tag_colors and has_highlights:
            value = f"<b>{TRANS_CFG['legend_label']}:</b>" + "".join(
                f'<span style="background-color:{color}; margin-left: 0.5em; color: black; padding: 0px 5px;">{label}</span>'
                for label, color in zip(cfg.tag_labels, cfg.tag_colors)
            )
        elif not value and not has_highlights:
            value = TRANS_CFG['legend_label_no_highlights']
        return gr.Markdown(value, visible=visible, elem_id="target_side_legend_cap")

    @classmethod
    def get_reload_btn(cls, visible: bool = False) -> gr.Button:
        return gr.Button(TRANS_CFG["reload_button_label"], variant="secondary", elem_id="reload_btn", visible=visible)

    @classmethod
    def get_done_btn(cls, visible: bool = False) -> gr.Button:
        return gr.Button(TRANS_CFG["done_button_label"], variant="primary", elem_id="done_btn", visible=visible)

    @classmethod
    def get_download_btn(cls, visible: bool = False) -> gr.DownloadButton:
        return gr.DownloadButton(
            TRANS_CFG["download_button_label"],
            variant="primary",
            elem_id="download_btn",
            visible=visible,
            interactive=False,
        )

    @classmethod
    def get_textboxes_col(cls, visible: bool = False) -> gr.Column:
        return gr.Column(visible=visible, elem_id="textboxes_col")

    @classmethod
    def get_textbox_txt(
        cls,
        type: Literal["source", "target"],
        id: int,
        value: str | Callable = "",
        visible: bool = False,
        lines: int = 2,
        has_highlights: bool = True
    ) -> gr.components.Textbox | HighlightedTextbox:
        if type == "source":
            return gr.Textbox(
                label=TRANS_CFG["source_textbox_label"],
                lines=lines,
                elem_id=f"{type}_{id}_txt",
                value=value,
                visible=visible,
                elem_classes=["source-textbox"],
                show_label=False,
            )
        elif type == "target":
            tuples = HighlightedTextbox.tagged_text_to_tuples(
                value,
                tag_ids=cfg.tag_labels,
                tags_open=[f"<{tag}>" for tag in cfg.allowed_tags],
                tags_close=[f"</{tag}>" for tag in cfg.allowed_tags],
            )
            color_map = None
            if cfg.tag_colors:
                if len(cfg.tag_colors) != len(cfg.tag_labels):
                    raise ValueError("highlight_colors and highlight_labels must have the same length")
                color_map = dict(zip(cfg.tag_labels, cfg.tag_colors))
            return HighlightedTextbox(
                value=tuples,
                label=TRANS_CFG["target_textbox_label"],
                elem_id=f"{type}_{id}_txt",
                interactive=True,
                show_label=False,
                show_legend=False,
                combine_adjacent=True,
                visible=visible,
                show_remove_tags_button=has_highlights,
                color_map=color_map,
            )

    @classmethod
    @buildmethod
    def build(
        cls: TranslateComponents,
        source_sentences: list[str] = [""] * cfg.max_num_sentences,
        target_sentences: list[str] = [""] * cfg.max_num_sentences,
    ) -> TranslateComponents:
        tc = TranslateComponents()
        with gr.Row(equal_height=True):
            tc.source_side_label = tc.get_source_side_label_cap()
            with gr.Column(visible=True):
                tc.target_side_label = tc.get_target_side_label_cap()
                tc.target_side_legend = tc.get_target_side_legend_cap()
        with tc.get_textboxes_col(visible=False) as textboxes_col:
            for idx, (src_sent, tgt_sent) in enumerate(zip(source_sentences, target_sentences)):
                with gr.Row(equal_height=True, visible=False):
                    _ = tc.get_textbox_txt("source", idx, src_sent, lines=0)
                    _ = tc.get_textbox_txt("target", idx, tgt_sent, lines=0)
        with gr.Row(equal_height=True):
            tc.reload_btn = tc.get_reload_btn()
            tc.done_btn = tc.get_done_btn()
        tc.download_btn = tc.get_download_btn()
        tc.textboxes_col = textboxes_col
        return tc

    def set_listeners(self, out_state: gr.State, lc_state: gr.State, logger: EventLogger) -> None:
        def save_logs_callback(state: dict[str, Any]) -> dict[str, Any]:
            if len(state["events"]) > cfg.event_logs_save_frequency:
                logger.save(state["events"])
                state["events"] = []
            return state

        def save_logs_callback_no_check(state: dict[str, Any]) -> dict[str, Any]:
            logger.save(state["events"])
            state["events"] = []
            return state

        for textbox in self.target_textboxes:
            textbox.focus(
                record_textbox_focus_fn,
                inputs=[out_state, textbox, lc_state],
                outputs=[out_state],
            )

            textbox.input(
                record_textbox_input_fn,
                inputs=[out_state, textbox, lc_state],
                outputs=[out_state],
                concurrency_limit=50,
                trigger_mode="multiple",
            )

            textbox.blur(
                record_textbox_blur_fn,
                inputs=[out_state, textbox, lc_state],
                outputs=[out_state],
            ).then(
                save_logs_callback,
                inputs=[out_state],
                outputs=[out_state],
            )

            textbox.clear(
                record_textbox_remove_highlights_fn,
                inputs=[out_state, textbox, lc_state],
                outputs=[out_state],
                trigger_mode="multiple",
            ).then(
                save_logs_callback,
                inputs=[out_state],
                outputs=[out_state],
            )

        self.done_btn.click(
            record_trial_end_fn,
            inputs=[out_state, lc_state],
            outputs=[out_state],
        ).then(
            save_logs_callback_no_check,
            inputs=[out_state],
            outputs=[out_state],
        ).then(
            save_outputs_to_file,
            inputs=[lc_state] + self.target_textboxes,
            outputs=[self.download_btn, self.done_btn],
        )



from __future__ import annotations

import csv
import json
import uuid
from logging import getLogger
from pathlib import Path
from typing import Any

import filelock
import huggingface_hub
from gradio import HuggingFaceDatasetSaver, utils

logger = getLogger(__name__)


class EventLogger(HuggingFaceDatasetSaver):
    """Override of HuggingFaceDatasetSaver to handle saving logged events in a state object."""

    def __init__(
        self,
        hf_token: str | None = None,
        dataset_id: str | None = None,
        private: bool = True,
        info_filename: str = "dataset_info.json",
        logging_dir: str = "logs",
        separate_dirs: bool = False,
    ):
        """
        Parameters:
            hf_token: The HuggingFace token to use to create (and write the logged sample to) the HuggingFace dataset
                (defaults to the registered one).
            dataset_id: The repo_id of the dataset to save the data to, e.g. "grote-logs" or "username/dataset-name".
            private: Whether the dataset should be private (defaults to True).
            info_filename: The name of the file to save the dataset info (defaults to "dataset_infos.json").
            logging_dir: The local directory where the dataset is cloned, updated, and pushed from (defaults to "logs")
                If no Hugging Face dataset is used, this is the only place where the data is saved.
            separate_dirs: If True, each logged item will be saved in a separate directory. This makes the logging
                more robust to concurrent editing, but may be less convenient to use.
        """
        self.hf_token = hf_token
        self.dataset_id = dataset_id
        self.dataset_private = private
        self.info_filename = info_filename
        self.separate_dirs = separate_dirs
        self.log_to_hf_dataset = hf_token is not None and dataset_id is not None
        if self.log_to_hf_dataset is None:
            logger.warning(
                "No Hugging Face token provided. Logging events will not be synchronized to the Hugging Face Hub."
            )
        else:
            huggingface_hub.login(token=self.hf_token, write_permission=True)
        self.setup(logging_dir)

    def setup(self, logging_dir: str):
        """
        Params:
            flagging_dir (str): local directory where the dataset is cloned, updated, and pushed from.
        """
        feature_names = ["time", "login_code", "text_id", "event_type", "text"]
        features = {name: {"dtype": "string", "_type": "Value"} for name in feature_names}
        self.features = features
        path_glob = "**/*.jsonl" if self.separate_dirs else "data.csv"

        # Setup logging dir
        self.dataset_dir = Path(logging_dir).absolute() / self.dataset_id.split("/")[-1]
        self.dataset_dir.mkdir(parents=True, exist_ok=True)
        self.infos_file = self.dataset_dir / self.info_filename

        # Setup dataset on the Hub
        if self.log_to_hf_dataset:
            self.dataset_id = huggingface_hub.create_repo(
                repo_id=self.dataset_id,
                token=self.hf_token,
                private=self.dataset_private,
                repo_type="dataset",
                exist_ok=True,
            ).repo_id
            huggingface_hub.metadata_update(
                repo_id=self.dataset_id,
                repo_type="dataset",
                metadata={
                    "configs": [
                        {
                            "config_name": "default",
                            "data_files": [{"split": "train", "path": path_glob}],
                        }
                    ]
                },
                overwrite=True,
                token=self.hf_token,
            )

            # Download remote files to local
            remote_files = [self.info_filename]
            if not self.separate_dirs:
                # No separate dirs => means all data is in the same CSV file => download it to get its current content
                remote_files.append("data.csv")

            for filename in remote_files:
                try:
                    huggingface_hub.hf_hub_download(
                        repo_id=self.dataset_id,
                        repo_type="dataset",
                        filename=filename,
                        local_dir=self.dataset_dir,
                        token=self.hf_token,
                    )
                except huggingface_hub.utils.EntryNotFoundError:
                    pass

    def save(self, data: list[dict[str, Any]]) -> int:
        if self.separate_dirs:
            # JSONL files to support dataset preview on the Hub
            unique_id = str(uuid.uuid4())
            data_dir = self.dataset_dir / unique_id
            data_file = data_dir / "metadata.jsonl"
            path_in_repo = unique_id  # upload in sub folder (safer for concurrency)
        else:
            # Unique CSV file
            data_dir = self.dataset_dir
            data_file = data_dir / "data.csv"
            path_in_repo = None  # upload at root level

        return self._save_in_dir(
            data_file=data_file,
            data_dir=data_dir,
            path_in_repo=path_in_repo,
            data=data,
        )

    def _save_in_dir(
        self,
        data_file: Path,
        data_dir: Path,
        path_in_repo: str | None,
        data: list[Any],
    ) -> int:
        # Deserialize components (write images/audio to files)
        rows = self._format_data(data)

        # Write generic info to dataset_infos.json + upload
        with filelock.FileLock(str(self.infos_file) + ".lock"):
            if not self.infos_file.exists():
                self.infos_file.write_text(json.dumps({"features": self.features}))

                if self.log_to_hf_dataset:
                    huggingface_hub.upload_file(
                        repo_id=self.dataset_id,
                        repo_type="dataset",
                        token=self.hf_token,
                        path_in_repo=self.infos_file.name,
                        path_or_fileobj=self.infos_file,
                    )

        headers = list(self.features.keys())

        if not self.separate_dirs:
            with filelock.FileLock(data_dir / ".lock"):
                sample_nb = self._save_as_csv(data_file, headers=headers, rows=rows)

                if self.log_to_hf_dataset:
                    sample_name = str(sample_nb)
                    huggingface_hub.upload_folder(
                        repo_id=self.dataset_id,
                        repo_type="dataset",
                        commit_message=f"Logged events up to #{sample_name}",
                        path_in_repo=path_in_repo,
                        ignore_patterns="*.lock",
                        folder_path=data_dir,
                        token=self.hf_token,
                    )
        else:
            sample_name = self._save_as_jsonl(data_file, headers=headers, rows=rows)

            if self.log_to_hf_dataset:
                sample_nb = len([path for path in self.dataset_dir.iterdir() if path.is_dir()])
                huggingface_hub.upload_folder(
                    repo_id=self.dataset_id,
                    repo_type="dataset",
                    commit_message=f"Logged events up to #{sample_name}",
                    path_in_repo=path_in_repo,
                    ignore_patterns="*.lock",
                    folder_path=data_dir,
                    token=self.hf_token,
                )

        return sample_nb

    def _format_data(self, data: list[dict[str, Any]]) -> tuple[dict[Any, Any], list[Any]]:
        """Standardize logged events for writing to disk."""
        return [[event[feature] if feature in event else None for feature in self.features.keys()] for event in data]

    @staticmethod
    def sanitize_list_for_csv(values: list[Any]) -> list[Any]:
        """
        Sanitizes a list of values (or a list of list of values) that is being written to a
        CSV file to prevent CSV injection attacks.
        """
        sanitized_values = []
        for value in values:
            if isinstance(value, list):
                sanitized_value = [utils.sanitize_value_for_csv(v) if v is not None else v for v in value]
            else:
                sanitized_value = utils.sanitize_value_for_csv(value) if value is not None else value
            sanitized_values.append(sanitized_value)
        return sanitized_values

    @classmethod
    def _save_as_csv(cls, data_file: Path, headers: list[str], rows: list[list[Any]]) -> int:
        """Save data as CSV and return the sample name (row number)."""
        is_new = not data_file.exists()

        with data_file.open("a", newline="", encoding="utf-8") as csvfile:
            writer = csv.writer(csvfile)

            # Write CSV headers if new file
            if is_new:
                writer.writerow(utils.sanitize_list_for_csv(headers))

            # Write CSV row for flagged sample
            writer.writerows(cls.sanitize_list_for_csv(rows))

        with data_file.open(encoding="utf-8") as csvfile:
            return sum(1 for _ in csv.reader(csvfile)) - 1

    @staticmethod
    def _save_as_jsonl(data_file: Path, headers: list[str], rows: list[list[Any]]) -> str:
        """Save data as JSONL and return the sample name (uuid)."""
        Path.mkdir(data_file.parent, parents=True, exist_ok=True)
        with open(data_file, "w") as f:
            save_dic = {}
            for i, header in enumerate(headers):
                save_dic[header] = [row[i] for row in rows]
            json.dump(save_dic, f)
        return data_file.parent.name



custom_css = """
.source-textbox textarea{
    user-select: none;
    cursor: not-allowed;
    pointer-events: none;
    border-width: 0px;
    resize: none;
}
.footer-custom-block {
    margin-top: 20px;
    display: flex;
    justify-content: center;
    align-items: center;
}
.footer-custom-block b {
    margin-right: 10px;
}
.footer-custom-block img {
    margin-right: 15px;
}
"""

ensure_dark_theme_js = """
function refresh() {
    const url = new URL(window.location);

    if (url.searchParams.get('__theme') !== 'dark') {
        url.searchParams.set('__theme', 'dark');
        window.location.href = url.href;
    }
}
"""



from __future__ import annotations

from pathlib import Path

import toml

from .app import make_demo


def get_version() -> str | None:
    pyproject_toml_file = Path(__file__).parent.parent / "pyproject.toml"
    if pyproject_toml_file.exists() and pyproject_toml_file.is_file():
        return toml.load(pyproject_toml_file)["tool"]["poetry"]["version"]
    return None


__version__ = get_version()

__all__ = ["make_demo"]



from __future__ import annotations

import os
from dataclasses import dataclass, field, fields
from pathlib import Path

import yaml


@dataclass
class GroteConfig:
    """Main configuration for the Grote application.

    Attributes:

    max_num_sentences: int
        The maximum number of sentences to be displayed at once in the translation tab. This is necessary
        since textboxes cannot be created dynamically in Gradio.
    login_codes: list[str]
        The list of valid login codes for the application.
    event_logs_save_frequency: int
        The frequency at which to save event logs expressed in number of events. Any change in the input of the
        textboxes will be logged as an event (e.g. one event per keystroke), and special events are stored for
        focus, blur, and removal of highlights.
    event_logs_hf_dataset_id: str
        The Hugging Face dataset ID to save the event logs to.
    event_logs_local_dir: str
        The local directory where the event logs are saved.
    hf_token: str | None
        The Hugging Face token to use to create (and write the logged sample to) the Hugging Face dataset
        (defaults to the registered one).
    allowed_tags: list[str]
        The list of allowed tags to be used in the translation tab.
    """

    max_num_sentences: int = 100
    login_codes: str | list[str] = "admin"
    event_logs_save_frequency: int = 50
    event_logs_hf_dataset_id: str = "grote-logs"
    event_logs_local_dir: str = "logs"
    hf_token: str | None = None
    allowed_tags: str | list[str] = field(default_factory=["minor", "major"])
    tag_labels: str | list[str] = field(default_factory=["Minor", "Major"])
    tag_colors: str | list[str] = field(default_factory=["#ffedd5", "#fcd29a"])

    @staticmethod
    def init_list(val: str | list) -> list:
        if isinstance(val, str):
            return val.split(",")
        return val

    @staticmethod
    def parse_int(val: str | int) -> int:
        if isinstance(val, str):
            return int(val)
        return val

    def __post_init__(self):
        self.max_num_sentences = self.parse_int(self.max_num_sentences)
        self.event_logs_save_frequency = self.parse_int(self.event_logs_save_frequency)
        self.login_codes = self.init_list(self.login_codes)
        self.allowed_tags = self.init_list(self.allowed_tags)
        self.tag_labels = self.init_list(self.tag_labels)
        self.tag_colors = self.init_list(self.tag_colors)


# Priority: environment variables > config.yaml
CONFIG = GroteConfig(
    **{
        **yaml.safe_load(open(Path(__file__).parent / "config.yaml", encoding="utf8")),
        **{k.lower(): v for k, v in os.environ.items() if k.lower() in [f.name.lower() for f in fields(GroteConfig)]},
    }
)



from pathlib import Path

import gradio as gr

from grote.collections import LoadComponents, TranslateComponents
from grote.config import CONFIG as cfg
from grote.event_logging import EventLogger
from grote.style import custom_css, ensure_dark_theme_js

event_logger = EventLogger(
    cfg.hf_token, cfg.event_logs_hf_dataset_id, private=True, logging_dir=cfg.event_logs_local_dir
)


def make_demo():
    with gr.Blocks(
        theme=gr.themes.Default(primary_hue="red", secondary_hue="pink"), css=custom_css, #js=ensure_dark_theme_js
    ) as demo:
        gr.HTML('<img src="file/assets/img/grote_logo.png" width=200px />')
        lc = LoadComponents.build()
        tc = TranslateComponents.build()
        out_state: gr.State = gr.State({"events": []})

        # Event Listeners
        tc.reload_btn.click(None, js="window.location.reload()")
        lc.set_listeners(tc, out_state)
        tc.set_listeners(out_state, lc.state, event_logger)
    return demo


demo = make_demo()


def main():
    current_file_path = Path(__file__).resolve()
    img_path = (current_file_path.parent / ".." / "assets" / "img").resolve()
    gr.set_static_paths(paths=[img_path])
    demo.queue(api_open=False).launch(show_api=False, allowed_paths=[img_path], favicon_path=img_path / "favicon.ico")


if __name__ == "__main__":
    main()



import re
from pathlib import Path
from typing import Any

import gradio as gr

from grote.config import CONFIG as cfg


def check_and_parse_inputs_fn(
    login_code_txt: str,
    file_in: gr.File,
    sentences_txt: str,
    state: dict[str, Any],
) -> None:
    """Checks if the loading inputs are valid.

    Args:
        login_code_txt (`str`):
            Textbox containing the login code.
        file_in (`gr.File`):
            File containing source and target sentences in `SRC ||| TGT` format.
        sentences_txt (`str`):
            Textbox containing source and target sentences in `SRC ||| TGT` format. Can be used instead of `file_in`.
        state (`dict`):
            The current state of the load tab in the application.

    Raises:
        `gr.Error`: Invalid login code.
        `gr.Error`: No sentences were provided in either `source_file_in` or `source_sentences_txt`.
        `gr.Error`: Sentences were provided in both `source_file_in` and `source_sentences_txt`.
        `gr.Error`: Sentences must match the format SOURCE ||| TARGET.
        `gr.Error`: Source sentences cannot contain highlights.
        `gr.Error`: Target sentences cannot contain unclosed highlights.
        `gr.Error`: Target sentences cannot contain invalid highlights (wrong order or wrong type)
    """
    if login_code_txt not in cfg.login_codes:
        raise gr.Error("ERROR: Invalid login code.")
    if not file_in and not sentences_txt:
        raise gr.Error("ERROR: No sentences were provided.")
    elif file_in and sentences_txt:
        raise gr.Error("ERROR: You can either upload a file or insert sentences manually, not both.")

    if file_in is not None:
        with open(file_in.name) as f:
            sentences_txt = f.read()
            # remove empty lines
            sentences_txt = re.sub(r"\n+", "\n", sentences_txt)
    sentences_txt = sentences_txt.strip().split("\n")

    if not all(" ||| " in s for s in sentences_txt):
        raise gr.Error("ERROR: Sentences must match the format SOURCE ||| TARGET")

    source_sentences = [sent.split(" ||| ")[0] for sent in sentences_txt]
    target_sentences = [sent.split(" ||| ")[1] for sent in sentences_txt]
    # Check wellformedness of source and target sentences (highlights allowed in target sentences only)
    find_tag_pattern = rf"(<\/?(?:{'|'.join(cfg.allowed_tags)})>)"
    open_tags = [f"<{tag}>" for tag in cfg.allowed_tags]
    close_tags = [f"</{tag}>" for tag in cfg.allowed_tags]
    source_matches = [(m.group(0),) + m.span() for m in re.finditer(find_tag_pattern, "\n".join(source_sentences))]
    if len(source_matches) > 0:
        raise gr.Error("ERROR: Source sentences cannot contain highlights.")
    state["_has_highlights"] = False
    for tgt_sent_idx, target_sentence in enumerate(target_sentences, start=1):
        target_matches = [(m.group(0),) + m.span() for m in re.finditer(find_tag_pattern, target_sentence)]
        num_matches = len(target_matches)
        if num_matches > 0:
            if num_matches % 2 != 0:
                raise gr.Error(f"ERROR: Target sentence {tgt_sent_idx} contains an unclosed highlight.")
            for curr_match_idx, match in enumerate(target_matches, start=1):
                if (curr_match_idx % 2 != 0 and match[0] not in open_tags) or (
                    curr_match_idx % 2 == 0 and match[0] not in close_tags
                ):
                    raise gr.Error(
                        f"ERROR: Target sentence {tgt_sent_idx} contains an invalid highlight ({curr_match_idx},"
                        f" {match[0]})."
                    )
            state["_has_highlights"] = True
    state["login_code_txt"] = login_code_txt
    state["file_in"] = None
    state["_filename"] = Path(file_in.name).stem if file_in is not None else "grote_sentences.txt"
    state["sentences_txt"] = sentences_txt
    return state


def initialize_translate_interface_fn(lc_state: dict[str, Any], tc_state: dict[str, Any]):
    """Initializes the translation tab."""
    from grote.collections import LoadComponents, TranslateComponents

    sentences = lc_state["sentences_txt"]
    source_sentences = [sent.split(" ||| ")[0] for sent in sentences]
    target_sentences = [sent.split(" ||| ")[1] for sent in sentences]
    num_sentences = len(sentences)
    lc_components = []
    for lc_elem_id in lc_state.keys():
        if not lc_elem_id.startswith("_"):
            lc_components.append(LoadComponents.make_component(lc_elem_id, visible=False))
    tc_components = []
    for tc_elem_id in tc_state.keys():
        if not tc_elem_id.startswith("_") and not tc_elem_id.endswith("_txt"):
            tc_components.append(TranslateComponents.make_component(tc_elem_id, visible=True, has_highlights=lc_state["_has_highlights"]))
    for tc_elem_id in [k for k in tc_state.keys() if k.endswith("_txt")]:
        txt_type, txt_id, _ = tc_elem_id.split("_")
        if int(txt_id) < len(source_sentences):
            if txt_type == "source":
                curr_sent = source_sentences[int(txt_id)]
            elif txt_type == "target":
                curr_sent = target_sentences[int(txt_id)]
            tc_components.append(TranslateComponents.get_textbox_txt(txt_type, txt_id, curr_sent, visible=True, has_highlights=lc_state["_has_highlights"]))
            tc_state[f"{txt_type}_{txt_id}_txt"] = curr_sent
        else:
            tc_components.append(TranslateComponents.get_textbox_txt(txt_type, txt_id, "", visible=False, has_highlights=lc_state["_has_highlights"]))
    n_hid = cfg.max_num_sentences - num_sentences
    return (
        [TranslateComponents.get_textboxes_col(visible=True)]
        + [lc_state]
        + lc_components
        + [tc_state]
        + tc_components
        + [gr.Row(visible=True) for _ in range(num_sentences)]
        + [gr.Row(visible=False) for _ in range(n_hid)]
    )



from abc import abstractmethod
from dataclasses import dataclass
from functools import wraps
from pathlib import Path
from typing import Callable, ClassVar, TypeVar

import gradio as gr
import yaml
import inspect
from gradio.blocks import BlockContext

T = TypeVar("T", bound="ComponentCollection")

NON_EDITABLE_COMPONENTS = [gr.Markdown, gr.Button]

COMPONENT_CONFIGS = yaml.safe_load(open(Path(__file__).parent / "config.yaml", encoding="utf8"))


def buildmethod(f: Callable[..., T]) -> Callable[..., T]:
    @wraps(f)
    def post_build_wrapper(cls, *args, **kwargs):
        component = f(cls, *args, **kwargs)
        component.post_build()
        return component

    return post_build_wrapper


@dataclass
class ComponentCollection:
    """A collection of Gradio components facilitating the creation of Gradio interfaces."""

    _state: gr.State = None
    _id: ClassVar[str] = "base"

    def __post_init__(self):
        pass

    def post_build(self):
        self._state = gr.State({c.elem_id: c.value for c in self.components})

    @classmethod
    def _get_components(cls, item) -> list[gr.components.Component]:
        """Recursive method to extract all Component from a given object"""
        if isinstance(item, gr.components.Component):
            return [item]
        elif isinstance(item, ComponentCollection):
            return item.list_components()
        elif isinstance(item, BlockContext):
            return [subc for c in item.children for subc in cls._get_components(c)]
        elif isinstance(item, list):
            return [subc for c in item for subc in cls._get_components(c)]
        elif isinstance(item, dict):
            return [subc for c in item for subc in cls._get_components(c)]
        else:
            return []

    @property
    def components(self) -> list[gr.components.Component]:
        all_components = []
        for f in self.__dataclass_fields__:
            component = getattr(self, f)
            all_components.extend(ComponentCollection._get_components(component))
        return all_components

    @property
    def editable_components(self) -> list[gr.components.Component]:
        return [c for c in self.components if not isinstance(c, NON_EDITABLE_COMPONENTS)]

    @property
    def state(self) -> gr.State:
        return self._state

    @state.setter
    def state(self, state: gr.State) -> None:
        self._state = state

    @classmethod
    def make_component(cls, elem_id: str, **kwargs) -> gr.components.Component:
        if not hasattr(cls, f"get_{elem_id}"):
            raise ValueError(f"Method get_{elem_id} not found for class {cls.__name__}.")
        builder_fn = getattr(cls, f"get_{elem_id}")
        # Filter kwargs to only pass those that are accepted by the builder function
        kwargs = {k: v for k, v in kwargs.items() if k in inspect.signature(builder_fn).parameters}
        return builder_fn(**kwargs)

    @classmethod
    @buildmethod
    def build(cls: T, **kwargs) -> T:
        raise NotImplementedError

    @abstractmethod
    def set_listeners(self, *args, **kwargs) -> None:
        raise NotImplementedError
